/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "e17genmenu.h"

static char *e17genmenu_opt = NULL;

void _e17genmenu_help() {
   printf("e17genmenu - Generate Enlightenment DR17 Menus From Gnome/Kde Menus\n");
   printf("Usage: e17genmenu <-options>\n");
	printf("\n");
	printf("To specify a theme, only pass the theme name NOT the full path.\n");
	printf("(ie: For the Hicolor theme, only specify hicolor, NOT /usr/share/icons/hicolor)\n\n");
	#ifdef HAVE_GNOME
	   printf(" -g | --gnome		Generate E17 Menus From Gnome Menus.\n");
		printf(" --gnome-theme=<theme>	Use Gnome icon theme <theme> for icons.\n");
	#endif
	#ifdef HAVE_KDE
	   printf(" -k | --kde		Generate E17 Menus From Kde Menus.\n");
		printf(" --kde-theme=<theme>	Use Kde icon theme <theme> for icons.\n");
	#endif
	printf(" -h | --help		Show this help screen.\n");	
}

void _e17genmenu_parseargs(int argc, char **argv) {
   int c;
	char *theme;
   static char *options = "gkh";
   static struct option long_options[] = {
		{"gnome", 0, 0, 'g'},
		{"gnome-theme", 1, 0, 0},
		{"kde", 0, 0, 'k'},
		{"kde-theme", 1, 0, 0},
		{"help", 0, 0, 'h'},
		{0, 0, 0, 0}
   };

	while ((c = getopt_long(argc, (char **) argv, options, long_options, NULL)) != -1) {
		switch (c) {
			#ifdef HAVE_GNOME
			case 'g':
		    	e17genmenu_opt = strdup("g");
				if (argc > 2) {
					if (strstr(argv[2], "gnome-theme") != NULL) {
						theme = parse_theme(strdup(argv[2]));
						if (theme) set_theme("Gnome", theme);
					}
				}
	    		break;
			#endif
			#ifdef HAVE_KDE
			case 'k':
		    	e17genmenu_opt = strdup("k");
				if (argc > 2) {
					if (strstr(argv[2], "kde-theme") != NULL) {
						theme = parse_theme(strdup(argv[2]));
						if (theme) set_theme("Kde", theme);
					}
				}
	    		break;
			#endif
			case 'h':
		    	e17genmenu_opt = strdup("h");
				break;
			default:
				break;
		}
	}
}

int main(int argc, char **argv) {
	char path[PATH_MAX];

	/* Parse Args */
	_e17genmenu_parseargs(argc, argv);
	
	ecore_app_args_set(argc, argv);    

	/* Setup Ecore_File */	
	if (!ecore_file_init()) {
		fprintf(stderr, "ERROR: Unable to init ecore_file, exiting\n");
		exit(-1);
    }

	/* Check That Dir All Exists */
	snprintf(path, PATH_MAX, "%s/.e/e/applications/all", get_home());
	if (!ecore_file_exists(path)) {
		fprintf(stderr, "ERROR: %s dosen't exist. Where are the eapps?\n", path);
		exit(-1);
	}

	/* Start */
	if (e17genmenu_opt != NULL) {
		if (strcmp(e17genmenu_opt, "g") == 0) {
			#ifdef HAVE_GNOME
				do_menus("Gnome");
			#endif
		} else if (strcmp(e17genmenu_opt, "k") == 0) {
			#ifdef HAVE_KDE
				do_menus("Kde");
			#endif
		} else if ((strcmp(e17genmenu_opt, "h") == 0) || (e17genmenu_opt == NULL)) {
			_e17genmenu_help();
			exit(-1);
		}
		
		/* Sort Menus */
		sort_menus();
		
		/* Shutdown Ecore */
		ecore_shutdown();
	} else {
		_e17genmenu_help();
		exit(-1);
	}
	return 0;
}

void do_menus(char *desk) {
   char path[PATH_MAX];
	char *home;
	
	home = get_home();
	
	fprintf(stderr, "Creating %s Menus\n", desk);

	/* Create Main_Menu Dir */	
	snprintf(path, sizeof(path), "%s"EFAVDIR"/%s", home, desk);
	create_dir(path);
	
	/* Create Main_Menu Dir Eap */
	create_dir_eap(path, desk);
	snprintf(path, sizeof(path), "%s"EFAVDIR, home);
	modify_order(path, desk);

	/* Create Sub_Dirs. NOTE: (Sub_Dir) Dir Eaps Are Created Inside Function */
	#ifdef HAVE_GNOME
		if (strcmp(desk, "Gnome") == 0) create_subdirs(STDDIRS, desk);
	#endif
	#ifdef HAVE_KDE
		if (strcmp(desk, "Gnome") == 0) create_subdirs(STDDIRS, desk);
		if (strcmp(desk, "Kde") == 0) create_subdirs(KDEDIRS, desk);
	#endif
	
	/* Generate Eaps */
	generate_eaps(desk);
}

char *parse_theme(char *arg) {
	char *c;

	c = strstr(arg, "=");
	c = strtok(c, "=");
	return c;
}
