/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "eap.h"

void write_eap(char *file, char *section, char *value) {
	int i;
	Eet_File *ef;

	#ifdef DEBUG
		fprintf(stderr, "Writting To Eap %s\n", file);
	#endif	
	eet_init();
	ef = eet_open(file, EET_FILE_MODE_READ_WRITE);
	if (!ef) {
		fprintf(stderr, "ERROR: Cannot open file %s for READ/WRITE\n", file);
		exit(-1);
   }
   if (strcmp(section, "app/info/startup_notify") == 0) {
   	if (value) {
   		i = atoi(value);
	   	eet_write(ef, section, &i, 1, 0);
	   } else {
		   eet_delete(ef, section);
	   }
   } else { 
		if (value) {
			eet_write(ef, section, value, strlen(value), 0);
		} else {
			eet_delete(ef, section);	
		}
	}
	eet_close(ef);
	eet_shutdown();
}
   
/* Create a .directory.eap for this dir */
void create_dir_eap(char *path, char *cat) {
	char path2[PATH_MAX];
	char *icon;
	
	snprintf(path2, sizeof(path2), "%s/.directory.eap", path);
	if (!ecore_file_exists(path2)) {
		icon = set_icon(cat);
		if (!icon) {
			fprintf(stderr, "ERROR: Cannot Find Icon For %s\n", cat);
			return;
		}
		write_icon(path2, icon);
		write_eap(path2, "app/info/name", cat);
	}
}

void create_subdirs(char *dirs, char *desk) {
	char d[PATH_MAX], path[PATH_MAX];
	char *home, *cat;
	
	home = get_home();
	snprintf(d, sizeof(d), strdup(dirs));
	cat = strtok(d, ":");
	while (cat) {
		/* Create Sub-Dir  & Dir Eap */
		snprintf(path, sizeof(path), "%s"EFAVDIR"/%s/%s", home, desk, cat);
		create_dir(path);
		create_dir_eap(path, cat);
		
		/* Add This Sub To Order File */
		snprintf(path, sizeof(path), "%s"EFAVDIR"/%s", home, desk);
		modify_order(path, cat);
		
		cat = strtok(NULL, ":");
	}
}

void create_dir(char *path) {
	if (!ecore_file_exists(path)) {
		fprintf(stderr, "\nCreating %s\n", path);
		if (!ecore_file_mkpath(path)) {
			fprintf(stderr, "ERROR: Cannot Make Path %s\n", path);
			exit(-1);
		}
	}
}

void generate_eaps(char *desk) {
	char path[PATH_MAX];
	
	if (strcmp(desk, "Gnome") == 0) {
		#ifdef HAVE_GNOME
			#ifdef DEBUG
				fprintf(stderr, "\nGenerating Gnome Eaps\n");
			#endif
			snprintf(path, sizeof(path), "%s", GNOMEAPPDIR);
			read_dir(desk, path);
			snprintf(path, sizeof(path), "%s", OLDGNOMEAPPDIR);
			read_dir(desk, path);
		#endif
	} else if (strcmp(desk, "Kde") == 0) {
		#ifdef HAVE_KDE
			#ifdef DEBUG
				fprintf(stderr, "\nGenerating Kde Eaps\n");
			#endif
			snprintf(path, sizeof(path), "%s", KDEAPPDIR);
			read_dir(desk, path);
			snprintf(path, sizeof(path), "%s", KDELNKDIR);
			read_dir(desk, path);
		#endif
	}
}
