/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "icons.h"

/* return directory from full path */
char *get_dir(char *icon) {
	char *ptr;
	char *d;
	char *dir;
	int i = 0;

	ptr = icon;
	d = strrchr(ptr, '/');
	if (!d) return ".";

	dir = malloc(strlen(icon) + 1);
	while (ptr != d) {
		dir[i] = *ptr;
		ptr++;
		i++;
	}
    dir[i] = '\0';
    return dir;
}

char *set_icon(char *token) {
	#ifdef HAVE_GNOME
		if (strstr(token, "Gnome") != NULL) return GNOMEICON;
	#endif
	#ifdef HAVE_KDE
		if (strstr(token, "Kde") != NULL) return KDEICON;
	#endif
	if ((strstr(token, "Accessories") != NULL) || (strstr(token, "Applications") != NULL))
		return APPLICATIONICON;
	if (strstr(token, "Core") != NULL)
		return COREICON; 
	if (strstr(token, "Development") != NULL)
		return PROGRAMMINGICON;
	if (strstr(token, "Editors") != NULL)
		return EDITORICON;
	if (strstr(token, "Edutainment") != NULL)
		return EDUTAINMENTICON;
	if (strstr(token, "Games") != NULL)
		return GAMESICON;
	if (strstr(token, "Graphics") != NULL)
		return GRAPHICSICON;
	if (strstr(token, "Internet") != NULL)
		return INTERNETICON;
	if ((strstr(token, "Multimedia") != NULL) || (strstr(token, "Sound_Video") != NULL))
		return MULTIMEDIAICON;
	if (strstr(token, "Office") != NULL)
		return OFFICEICON;
	if (strstr(token, "Programming") != NULL)
		return PROGRAMMINGICON;
	if ((strstr(token, "Preferences") != NULL) || (strstr(token, "Settings") != NULL))
		return SETTINGSICON;
	if ((strstr(token, "System") != NULL) || (strstr(token, "System_Tools") != NULL))
		return SYSTEMICON;
	if (strstr(token, "Toys") != NULL)
		return TOYSICON;
	if (strstr(token, "Utilities") != NULL)
		return UTILITYICON;
}

void write_icon(char *f, char *i) {
	Engrave_File *eet;
   Engrave_Image *image;
   Engrave_Group *grp;
   Engrave_Part *part;
   Engrave_Part_State *ps;
   char *idir, *ifile;

	#ifdef DEBUG
		fprintf(stderr, "Creating %s\nUsing Icon %s\n", f, i);
	#endif
   ifile = strrchr(i, '/');
   idir = ecore_file_get_dir(i);
	
   eet = engrave_file_new();
	engrave_file_image_dir_set(eet, idir);
  	image = engrave_image_new(ifile, ENGRAVE_IMAGE_TYPE_COMP, 0);
  	engrave_file_image_add(eet, image);

   grp = engrave_group_new();
   engrave_group_name_set(grp, "icon");
   engrave_group_max_size_set(grp, 48, 48);
   engrave_file_group_add(eet, grp);

   part = engrave_part_new(ENGRAVE_PART_TYPE_IMAGE);
   engrave_part_name_set(part, "image");
   engrave_part_mouse_events_set(part, 0);
   engrave_group_part_add(grp, part);

   ps = engrave_part_state_new();
   engrave_part_state_name_set(ps, "default", 0.0);
   engrave_part_state_aspect_set(ps, 1.0, 1.0);
   engrave_part_state_image_normal_set(ps, image);
   engrave_part_state_add(part, ps);

   engrave_eet_output(eet, f);
   engrave_file_free(eet);
}

char *find_icon(char *desk, char *icon) {
	char icn[PATH_MAX], dirs[PATH_MAX], p[PATH_MAX];
	char apps[PATH_MAX], devs[PATH_MAX], fs[PATH_MAX];
	char stock[PATH_MAX];
	char *d, *dir, *theme;
	int ret;

	if (icon == NULL) return APPLICATIONICON;

	snprintf(icn, sizeof(icn), "%s", icon);

	if ((strcmp(icon + strlen(icon) - 4, ".svg") == 0) || (strcmp(icon + strlen(icon) - 4, ".ico") == 0)) {
		#ifdef DEBUG
			fprintf(stderr, "Unsupported Icon Format !!\nUsing Default Icon\n");	
		#endif
		return APPLICATIONICON;
	}
	
	/* Check For An Extension, Append PNG If Missing */
	if (strrchr(icon, '.') == NULL) snprintf(icn, sizeof(icn), "%s.png", icon);

	#ifdef DEBUG
		fprintf(stderr, "Checking For Icon %s\n", icn);
	#endif

	/* Seperate Filename, Get Dir Only */
	d = get_dir(icn);
	
	/* If Dir Was Supplied, Search It */
	if (strcmp(d, ".") != 0) {
		if (ecore_file_exists(icn)) {
			fprintf(stderr, "Found Icon %s\n", icn);
			return strdup(icn);
		}
	} else if (strstr(d, ".")) {
			/* No DIR Supplied, or Icon Not Found Above */

			/* Check For Theme If Specified */
			#ifdef DEBUG
				fprintf(stderr, "Getting Theme\n");
			#endif
			theme = (char *)get_theme(desk);
			if (theme != NULL) {
				fprintf(stderr, "Using Icon Theme %s\n", theme);
			} else {
				fprintf(stderr, "No Theme Supplied\n");
			}
			
			#ifdef HAVE_GNOME
			if (strstr(desk, "Gnome") != NULL) {
				if (theme != NULL) {
					snprintf(apps, sizeof(apps), "%s/share/icons/%s/48x48/apps", GNOME_PREFIX, theme);
					snprintf(devs, sizeof(devs), "%s/share/icons/%s/48x48/devices", GNOME_PREFIX, theme);
					snprintf(fs, sizeof(fs), "%s/share/icons/%s/48x48/filesystems", GNOME_PREFIX, theme);
					snprintf(dirs, sizeof(dirs), "%s:%s:%s", apps, devs, fs);
				} else {
					snprintf(dirs, sizeof(dirs), "%s:%s:%s", GNOMEAPPICONDIR, GNOMEDEVSICONDIR, GNOMEFILESICONDIR);
				}
			}
			#endif
			#ifdef HAVE_KDE
			if (strstr(desk, "Kde")) {
				if (theme != NULL) {
					snprintf(apps, sizeof(apps), "%s/share/icons/%s/32x32/apps", KDE_PREFIX, theme);
					snprintf(devs, sizeof(devs), "%s/share/icons/%s/32x32/devices", KDE_PREFIX, theme);
					snprintf(fs, sizeof(fs), "%s/share/icons/%s/32x32/filesystems", KDE_PREFIX, theme);
					snprintf(dirs, sizeof(dirs), "%s:%s:%s", apps, devs, fs);
				} else {
					snprintf(dirs, sizeof(dirs), "%s:%s:%s", KDEICONDIR, KDEFILESICONDIR, KDEDEVSICONDIR);
				}
			}
			#endif

			dir = strtok(dirs, ":");
			while (dir) {
				snprintf(p, sizeof(p), "%s/%s", dir, icn);
				#ifdef DEBUG
					fprintf(stderr, "Icon Path Is %s\n", p);
				#endif
				ret = ecore_file_exists(p);
				if (ret) break;
				dir = strtok(NULL, ":");
			}
		}
	
	
	/* No Icon Found Yet, Check Pixmap Dir */
	if (!ret) {
		snprintf(p, sizeof(p), "%s/%s", PIXMAPDIR, icn);
		ret = ecore_file_exists(p);
	}
	/* Return A Default Icon If None Found */
	if (!ret) {
		fprintf(stderr, "Icon Not Found %s\n", icn);
		return APPLICATIONICON;
	} else {
		fprintf(stderr, "Found Icon %s\n", p);
		return strdup(p);
	}	
}
