/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
#include "menus.h"

char *get_eap_name(char *file) {
	char *tmp, *tmp2;
	char eap[PATH_MAX];
	
	tmp2 = ecore_file_get_file(file);
	tmp = strtok(tmp2, ".");
	snprintf(eap, sizeof(eap), "%s.eap", tmp);	
	return strdup(eap);
}
        
void read_dir(char *desk, char *path) {
	int id;
	char *item;
	Ecore_List *list = NULL;
	
	#ifdef DEBUG
		fprintf(stderr, "Reading Dir %s\n", path);
	#endif
	
	/* Make Sure Path Exists */
	if (!ecore_file_exists(path)) {
		fprintf(stderr, "ERROR: %s does not exist\n\n", path);
		exit(-1);
	}
	
	/* Get List Of Files In This Dir */
	list = ecore_file_ls(path);
	if (!list) {
		fprintf(stderr, "ERROR: ecore_file_ls Failed For Dir %s!!\n", path);
		exit(-1);
	}

	/* For Each Entry In List, Read File */
	ecore_list_goto_first(list);
	ecore_list_for_each(list, read_file, path);

	/* Free List */
	ecore_list_destroy(list);
}

/* User_Data holds path to file */
void read_file(void *value, void *user_data) {
	char *eap, *home;
	char path2[PATH_MAX];
	
	#ifdef DEBUG
		fprintf(stderr, "Reading File %s\n", value);
	#endif
	if (strcmp(value + strlen(value) - 8, ".desktop") == 0) {
		if (strcmp(value, "panel_appearance.desktop") == 0) return;
		if (strcmp(value, "ktip.desktop") == 0) return;
		if (strcmp(value, "kalarm.desktop") == 0) return;

		/* Get Eap Name From Desktop File */
		eap = get_eap_name(strdup(value));
		#ifdef DEBUG
			fprintf(stderr, "Eap Name Is %s\n", eap);
		#endif
		
		/* Assemble Eap Path */
		home = get_home();
		snprintf(path2, sizeof(path2), "%s"EAPPDIR"/%s", home, eap);
		
		/* Check If Eap Exists */
//		if (!ecore_file_exists(path2)) {
			#ifdef DEBUG
				fprintf(stderr, "Parsing Desktop File: %s\n", value);
			#endif
			parse_file(user_data, value);
//		} else {
			/* Eap Exists, Overwrite Flag Given. User may be changing icon theme */
//			if (overwrite == 1) parse_file(user_data, value);				
//		}
	}
}
