/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "order.h"

void create_order(char *path) {
	FILE *f;

	#ifdef DEBUG
		fprintf(stderr, "Creating Order File %s\n", path);
	#endif
	f = fopen(path, "w+");
	if (!f) {
		fprintf(stderr, "ERROR: Cannot Create Order File %s\n", path);
		exit(-1);
	} else {
		fclose(f);
	}
}

void modify_order(char *path, char *entry) {
	int length, i;
	char buffer[PATH_MAX], path2[PATH_MAX];
   FILE *f;
	Evas_List *list = NULL;

	/* Append .order to Path */
	snprintf(path2, sizeof(path2), "%s/.order", path);

	#ifdef DEBUG
		fprintf(stderr, "Modifying Order File %s\n", path2);
	#endif
	
	entry = (char*)ecore_file_get_file(entry);
	fprintf(stderr, "Adding Eap %s\n", entry);		
	
	/* Stat .order; Create If Not Found */
	if (!ecore_file_exists(path2)) {
		create_order(path2);
		/* If We Had To Create This Order Then Just Add The Entry */
		list = evas_list_append(list, entry);
	} else {
		/* Open .order File For Parsing */
		f=fopen(path2, "r");
		if (!f) {
			fprintf(stderr, "ERROR: Cannot Open Order File %s \n", path2);
			exit(-1);
		}
		
		/* Read All Entries From Existing Order File, Store In List For Sorting*/
		while (fgets(buffer, sizeof(buffer), f) != NULL) {
			/* Strip New Line Char */
			if (buffer[(length = strlen(buffer) - 1)] == '\n') buffer[length] = '\0';
			/* ****************************************************
				TODO: Determine If Following Search Is Needed
				**************************************************** */
			/* Search Through List To See If Is Existing */
			//if (!search_list(list, buffer)) {
				/* Not Found, Add It To List */
				list = evas_list_append(list, strdup(buffer));
				if (evas_list_alloc_error()) {
					fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
					if (list != NULL) list = evas_list_free(list);
					exit(-1);
				}			
			//}
		}
		fclose(f);	

		buffer[0] = (char) 0;

		/* Add This Entry To List Of Existing ? */
		if (!search_list(list, entry)) {
			list = evas_list_append(list, entry);
			if (evas_list_alloc_error()) {
				fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
				if (list != NULL) list = evas_list_free(list);
				exit(-1);
			}			
		}
	}

	if (evas_list_count(list) > 1) {
		/* Add This Entry To List ? */
		if (!search_list(list, entry)) {
			list = evas_list_append(list, entry);
			if (evas_list_alloc_error()) {
				fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
				if (list != NULL) list = evas_list_free(list);
				exit(-1);
			}			
		}
	}

	#ifdef DEBUG
		fprintf(stderr, "Sorting List\n");
	#endif

	/* Sort List */
	list = evas_list_sort(list, evas_list_count(list), sort_list);
	if (evas_list_alloc_error()) {
		fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
		if (list != NULL) list = evas_list_free(list);
		exit(-1);
	}
	
	#ifdef DEBUG
		fprintf(stderr, "Rewritting Order %s\n", path2);
	#endif
	
	/* List Is Sorted, rewrite .order */
	if (!ecore_file_exists(path2)) create_order(path2);
	f = fopen(path2, "w+");
	if (!f) {
		fprintf(stderr, "ERROR: Cannot Open Order File %s \n", path2);
		if (list != NULL) list = evas_list_free(list);
		exit(-1);
	}

	for (i = 0; i < evas_list_count(list); i++) {
		snprintf(buffer, sizeof(buffer), "%s\n", strdup(evas_list_nth(list, i)));
		fwrite(buffer, sizeof(char), strlen(buffer), f);
	}
	fclose(f);
	
	if (list != NULL) list = evas_list_free(list);
}
