/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "parse.h"

char *get_desk() {
	int argc, c;
	char **argv;
	
	ecore_app_args_get(&argc, &argv);
	if (strcmp(argv[1], "-g") == 0) return strdup("Gnome");
	if (strcmp(argv[1], "--gnome") == 0) return strdup("Gnome");
	if (strcmp(argv[1], "-k") == 0) return strdup("Kde");
	if (strcmp(argv[1], "--kde") == 0) return strdup("Kde");
}

void parse_file(char *path, char *fl) {
	char buffer[PATH_MAX], e[PATH_MAX], orderpath[PATH_MAX], desk_file[PATH_MAX];
	char *token, *oldtoken, *name, *generic, *comment, *exec;
	char *category, *startup, *icon, *desk, *home, *eap;
	int i, length, process = 0;
	FILE *f;
	Evas_List *list = NULL;
	
	snprintf(desk_file, sizeof(desk_file), "%s/%s", path, fl);
	#ifdef DEBUG
		fprintf(stderr, "Path Is %s\n", desk_file);
	#endif
	f=fopen(desk_file, "r");
	if (!f) {
		fprintf(stderr, "ERROR: Cannot Open File %s \n", desk_file);
		return;
	}

	/* Read All Entries, Store In List */
	while (fgets(buffer, sizeof(buffer), f) != NULL) {
		/* Strip New Line Char */
		if (buffer[(length = strlen(buffer) - 1)] == '\n') buffer[length] = '\0';
			list = evas_list_append(list, strdup(buffer));
			if (evas_list_alloc_error()) {
				fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
				exit(-1);
			}			
	}
	fclose(f);
	
	buffer[0] = (char) 0;
	
	/* Loop through list and find out if we process this one */
	for (i = 0; i < evas_list_count(list); i++) {
		if (strstr(evas_list_nth(list, i), "Type=") != 0) {
			token = strtok(strdup(evas_list_nth(list, i)), "=");
			while (token) {
				token = strstr(token, "=");
				token = strtok(token, "=");
				if (token != NULL) {
					if (strcmp(token, "Application") == 0) process = 1;
				}
				token = strtok(NULL, "=");
			}
		}
	}			

	/* If we process this one, do it */
	if (process) {
		#ifdef DEBUG
			fprintf(stderr, "Processing %s\n", desk_file);
		#endif
		for (i = 0; i < evas_list_count(list); i++) {
			oldtoken = evas_list_nth(list, i);
			token = strtok(strdup(evas_list_nth(list, i)), "=");
			while (token) {
				token = strstr(token, "=");
				token = strtok(token, "=");
				if (token != NULL) {
					if (strncmp(oldtoken, "Name=", 5) == 0) name = strdup(token);
					if (strncmp(oldtoken, "GenericName=", 12) == 0) generic = strdup(token);
					if (strncmp(oldtoken, "Comment=", 8) == 0) comment = strdup(token);
					if (strncmp(oldtoken, "Categories=", 11) == 0) category = strdup(token);
					if (strncmp(oldtoken, "Exec=", 5) == 0) exec = strdup(token);
					if (strncmp(oldtoken, "Icon=", 5) == 0) icon = strdup(token);
					if (strncmp(oldtoken, "StartupNotify=", 14) == 0 || strncmp(oldtoken, "X-KDE-StartupNotify=", 20) == 0) {
						startup = (strcmp(token, "true") == 0) ? "1" : "0";
					}
				}
				token = strtok(NULL, "=");
			}
		}

		home = get_home();
		desk = get_desk();
		if (desk == NULL) return;
		eap = get_eap_name(desk_file);
		
		snprintf(e, sizeof(e), "%s"EAPPDIR"/%s", home, eap);
		icon = (icon != NULL) ? find_icon(desk, strdup(icon)) : strdup(APPLICATIONICON);
		
		write_icon(e, icon);
		write_eap(e, "app/info/name", name);

		if (generic != NULL) write_eap(e, "app/info/generic", generic);
		if (comment != NULL) write_eap(e, "app/info/comments", comment);

		if (exec != NULL) exec = parse_exec(exec);
		if (exec != NULL) write_eap(e, "app/info/exe", exec);

		if (startup != NULL) write_eap(e, "app/info/startup_notify", startup);

		char *window_class;
		if (ecore_file_exists(e)) {
			/* Get WM_CLASS From Eapp */
			window_class = get_window_class(e);	
		}

		if (category != NULL) category = find_category(category);
		if (category != NULL) {
			snprintf(orderpath, sizeof(orderpath), "%s"EFAVDIR"/%s/%s", home, desk, category);
			modify_order(orderpath, e);
		}
	}
	/* Clean Up List */
	list = evas_list_free(list);
}

char *parse_exec(char *exec) {
	char *token;

	token = strdup(exec);
	/* Strip Caption From Exec */	
	if (strstr(token, "caption") != NULL) {
		token = strdup(token);
		token = strtok(token, " ");
	}

	/* Strip %U from Exec */
	if (strstr(token, "%") != NULL) {
		token = strdup(token);
		token = strtok(token, " ");
	}
	return strdup(token);
}

char *get_window_class(char *eap) {
	char *tmp;
	int i;
	Eet_File *ef;

	if (ecore_file_exists(eap)) {
		#ifdef DEBUG
			fprintf(stderr, "Reading From Eap %s\n", eap);
		#endif	
		eet_init();
		ef = eet_open(eap, EET_FILE_MODE_READ);
		if (!ef) {
			fprintf(stderr, "ERROR: Cannot open file %s for READ/WRITE\n", eap);
			exit(-1);
	   }
		tmp = eet_read(ef, "app/info/win_class", i);
		if (!tmp) fprintf(stderr, "Cannot Read Window Class\n");
		eet_close(ef);
		eet_shutdown();
		return tmp;
	}
}
