/*
 * Copyright (C) 2005 Chris Michael
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "sort.h"
#include "global.h"

char *home;

char *getname(char *path) {
	Eet_File *ef;
	int ret;
	char *n, *nbuff, *name;
	
	eet_init();
	ef = eet_open(path, EET_FILE_MODE_READ);
	if (!ef) {
		fprintf(stderr, "\n\nERROR: Cannot Open File %s for READ\n", path);
		return (NULL);
   }
	n = (char *) eet_read(ef, "app/info/name", &ret);
   nbuff = (char *) malloc(ret + 1);
   snprintf(nbuff, ret + 1, "%s", n);
	name = strdup(nbuff);
	
	eet_close(ef);
	eet_shutdown();

	free(nbuff);
	return name;
}

void sort_menu(char *path) {
	char path2[PATH_MAX];
	char buffer[PATH_MAX];
	char entry[PATH_MAX];
	FILE *file;
	Evas_List *names = NULL;
	Evas_List *eaps = NULL;
	Evas_List *order = NULL;
	int i, j, length;
	char *name, *t, *u, *token;
	
	
	snprintf(path2, sizeof(path2), "%s/.order", path);
	if (!ecore_file_exists(path2)) return;
	
	file = fopen(path2, "r");
	if (!file) {
		fprintf(stderr, "ERROR: Cannot Open %s\n", path2);
		return;
	}

	home = get_home();
	while (fgets(buffer, sizeof(buffer), file) != NULL) {
		if (buffer[(length = strlen(buffer) - 1)] == '\n') buffer[length] = '\0';
		snprintf(path2, sizeof(path2), "%s"EAPPDIR"/%s", home, buffer);
		name = getname(path2);
		if (name) {
			snprintf(entry, sizeof(entry), "%s:%s", name, buffer);
			if (!search_list(names, name)) {
				names = evas_list_append(names, strdup(name));
				if (evas_list_alloc_error()) {
					fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
					return;
				}		
			}
			if (!search_list(eaps, entry)) {
				eaps = evas_list_append(eaps, strdup(entry));
				if (evas_list_alloc_error()) {
					fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
					names = evas_list_free(names);
					return;
				}		
			}
		}
	}
	fclose(file);
	
	names = evas_list_sort(names, evas_list_count(names), sort_list);
	eaps = evas_list_sort(eaps, evas_list_count(eaps), sort_list);
	for (i=0; i < evas_list_count(names); i++) {
		t = evas_list_nth(names, i);
		for (j=0; j < evas_list_count(eaps); j++) {
			u = evas_list_nth(eaps, j);
			if (strstr(u, t)) {
				token = strtok(u, ":");
				while (token) {
					if (token != u) {
						order = evas_list_append(order, strdup(token));
						if (evas_list_alloc_error()) {
							fprintf(stderr, "ERROR: Evas List Allocation Failed !!\n");
							names = evas_list_free(names);
							eaps = evas_list_free(eaps);
							return;
						}		
					}
					token = strtok(NULL, ":");
				}
			}
		}
	}
	
	/* Rewrite .order file with sorted order list */
	snprintf(path2, sizeof(path2), "%s/.order", path);
	file = fopen(path2, "w");
	if (!file) {
		fprintf(stderr, "ERROR: Cannot Open %s\n", path2);
		names = evas_list_free(names);
		eaps = evas_list_free(eaps);
		order = evas_list_free(order);
		return;
	}
	for (i = 0; i < evas_list_count(order); i++) {
		fputs(evas_list_nth(order, i), file);
		fputs("\n", file);	
	}	
	fclose(file);
	
	names = evas_list_free(names);
	eaps = evas_list_free(eaps);
	order = evas_list_free(order);
}
  
void sort_menus() {
	DIR *dir;
	DIR *dir2;
	struct dirent *entry;
	struct dirent *entry2;
	struct stat st;
	char path[PATH_MAX];
	
	home = get_home();
	snprintf(path, sizeof(path), "%s"EFAVDIR, home);
	dir = opendir(path);
	if (!dir) return;
	while (entry = readdir(dir)) {
		if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) continue;
		snprintf(path, sizeof(path), "%s"EFAVDIR"/%s", home, entry->d_name);
  		stat(path, &st);
  		if (S_ISDIR(st.st_mode)) {
  			dir2 = opendir(path);
  			if (dir2) {
  				while (entry2 = readdir(dir2)) {
					if (!strcmp(entry2->d_name, ".") || !strcmp(entry2->d_name, "..")) continue;
					snprintf(path, sizeof(path), "%s"EFAVDIR"/%s/%s", home, entry->d_name, entry2->d_name);
					stat(path, &st);
					if (S_ISDIR(st.st_mode)) sort_menu(path);					
				}
  			}
  		}
	}
	closedir(dir);	
}
